
#ifndef KGAPIPEOPLE_EXPORT_H
#define KGAPIPEOPLE_EXPORT_H

#ifdef KGAPIPEOPLE_STATIC_DEFINE
#  define KGAPIPEOPLE_EXPORT
#  define KGAPIPEOPLE_NO_EXPORT
#else
#  ifndef KGAPIPEOPLE_EXPORT
#    ifdef KPim6GAPIPeople_EXPORTS
        /* We are building this library */
#      define KGAPIPEOPLE_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KGAPIPEOPLE_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KGAPIPEOPLE_NO_EXPORT
#    define KGAPIPEOPLE_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KGAPIPEOPLE_DEPRECATED
#  define KGAPIPEOPLE_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KGAPIPEOPLE_DEPRECATED_EXPORT
#  define KGAPIPEOPLE_DEPRECATED_EXPORT KGAPIPEOPLE_EXPORT KGAPIPEOPLE_DEPRECATED
#endif

#ifndef KGAPIPEOPLE_DEPRECATED_NO_EXPORT
#  define KGAPIPEOPLE_DEPRECATED_NO_EXPORT KGAPIPEOPLE_NO_EXPORT KGAPIPEOPLE_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KGAPIPEOPLE_NO_DEPRECATED
#    define KGAPIPEOPLE_NO_DEPRECATED
#  endif
#endif

#endif /* KGAPIPEOPLE_EXPORT_H */
